<?php

namespace FiloBlu\Refilo\Model\Indexer\Processor\Type;

use Magento\Catalog\Model\Product;
use stdClass;

abstract class AbstractType
{

    /**
     * @var array
     */
    protected $_swatch_options;

    /**
     * @var array
     */
    protected $_options;

    /**
     * @var Product
     */
    protected $_product;
    /**
     * @var stdClass
     */
    protected $_item;

    /**
     * @var array
     */
    protected $_product_attributes;

    protected $allowedInputTypes = [
        'boolean',
        'multiselect',
        'select',
    ];

    protected $excludedAttributes = [
        'allow_message',
        'allow_open_amount',
        'created_at',
        'custom_design',
        'custom_design_from',
        'custom_design_to',
        'custom_layout',
        'custom_layout_update',
        'custom_layout_update_file',
        'email_template',
        'has_options',
        'media_gallery',
        'minimal_price',
        'price',
        'msrp_display_actual_price_type',
        'old_id',
        'open_amount_max',
        'open_amount_min',
        'image',
        'image_label',
        'links_exist',
        'links_purchased_separately',
        'links_title',
        'merchant_center_category',
        'msrp',
        'open_amount_max',
        'open_amount_min',
        'options_container',
        'origin_storage',
        'package_mode',
        'package_type',
        'page_layout',
        'price_type',
        'price_view',
        'quantity_and_stock_status',
        'related_tgtr_position_behavior',
        'related_tgtr_position_limit',
        'required_options',
        'small_image',
        'small_image_label',
        'special_price',
        'status',
        'swatch_image',
        'thumbnail',
        'thumbnail_label',
        'tier_price',
        'type_of_product',
        'updated_at',
        'upsell_tgtr_position_behavior',
        'upsell_tgtr_position_limit',
        'use_config_allow_message',
        'use_config_email_template',
        'use_config_is_redeemable',
        'visibility',
        'shipment_type'
    ];

    /**
     * @return array
     */
    public function getProductAttributes()
    {
        return $this->_product_attributes;
    }

    /**
     * @param array $product_attributes
     */
    public function setProductAttributes($product_attributes)
    {
        $this->_product_attributes = $product_attributes;
    }

    /**
     * @return mixed
     */
    public function getProduct()
    {
        return $this->_product_collection;
    }

    /**
     * @param mixed $product_collection
     */
    public function setProduct($product)
    {
        $this->_product = $product;
    }

    /**
     * @return stdClass
     */
    public function getItem()
    {
        return $this->_item;
    }

    /**
     * @return array
     */
    public function getSwatchOptions()
    {
        return $this->_swatch_options;
    }

    /**
     * @param array $swatch_options
     */
    public function setSwatchOptions($swatch_options)
    {
        $this->_swatch_options = $swatch_options;
    }

    /**
     * @return array
     */
    public function getOptions(): array
    {
        return $this->_options;
    }

    /**
     * @param array $options
     */
    public function setOptions(array $options): void
    {
        $this->_options = $options;
    }

    abstract protected function process();

    abstract protected function processBasedata();

    abstract protected function processAssets();

    abstract protected function processAttributes();

    abstract protected function processFilters($attribute);

    abstract protected function processSortables($attribute);

    abstract protected function processCategories();

    abstract protected function processPrices();

    abstract protected function processInventory();
}
