<?php

namespace FiloBlu\Refilo\Remote\Connector;

use Elasticsearch\Client;
use Elasticsearch\ClientBuilder;
use phpDocumentor\Reflection\Types\String_;
use Traversable;

/**
 * Class ElasticSearchConnector
 * @package FiloBlu\Refilo\Remote\Connector
 */
class BumbleSearch extends AbstractConnector
{
    /**
     * @var Client
     */
    protected $handler;

    /**
     * @var string
     */
    protected $username;

    /**
     * @var string
     */
    protected $password;

    /**
     * @var array
     */
    protected $base_mapping = [
        [
            "path" => 'assets.*',
            "analyzer" => "disabled"
        ],
        [
            "path" => 'crosssells',
            "analyzer" => "disabled"
        ],
        [
            "path" => "inventory.*",
            "analyzer" => "disabled"
        ],
        [
            "path" => "meta_*",
            "analyzer" => "disabled"
        ],
        [
            "path" =>  "rating.*",
            "analyzer" => "disabled"
        ],
        [
            "path" => "related",
            "analyzer" => "disabled"
        ],
        [
            "path" => "upsells",
            "analyzer" => "disabled"
        ],
        [
            "path" => "variants.*",
            "analyzer" => "disabled"
        ],
        [
            "path" => "bundle.*",
            "analyzer" => "disabled"
        ]
    ];

//    /**
//     * @var string
//     */
//    protected $auth_url = "https://welcome-ph3ofm4rwa-ew.a.run.app";

    /**
     * @return mixed|void
     */
    public function disconnect(): ConnectorInterface
    {
        if (!$this->handler) {
            return $this;
        }

        $this->handler = null;

        return $this;
    }

    /**
     * @param $entity
     */
    public function cleanDoc($entity) {

        $doc = $entity->toArray();
        $doc['id'] = (string)$entity->getId();

//        if (isset($doc['filters'])) {
//            foreach ($doc['filters'] as $key => $filter) {
//                $doc['attributes'][$key] = $filter;
//            }
//            unset($doc['filters']);
//            unset($doc['sortables']);
//        }

        return $doc;

    }

    /**
     * @param Traversable $entities
     * @param string $collection
     * @return mixed|void
     */
    public function create(Traversable $entities, string $collection, $action = 'insert')
    {

        if ($this->metadata && $this->metadata->getDataByKey('name') == 'attributes') {
            return  $this->mapFields($collection,$entities);
        }

        if ($action == 'insert') {
            $this->createIndex($collection);
        }

        $client = $this->getHandler();

        $i = 1;

        try {

            foreach ($entities as $entity) {

                $doc = $this->cleanDoc($entity);

                $docs[] = $doc;

                if ($i % 500 === 0) {
                    $response = $client->request('POST', 'indexes/'.$collection.'/bulk?action='.$action, [
                        'json'    => $docs,
                        'headers' => $this->getHeaders()
                    ]);
                    $docs = [];
                }
                $i++;


            }

            if (!empty($docs)) {
                $response = $client->request('POST', 'indexes/'.$collection.'/bulk?action='.$action, [
                    'json'    => $docs,
                    'headers' => $this->getHeaders()
                ]);
            }

        } catch (\Exception $e) {

            echo $e->getMessage();

        }


    }

    private function buildMapping($fields, $entities) {

        $mapping = [];

        foreach ($entities as $entity) {

            $included_attr = ($entity->getData('isFilterable') || $entity->getData('isSortable') || ($entity->getData('searchable')));

            if ($entity->getData('code') === 'price' || !$included_attr)
                continue;

            $attribute_prefix = ($entity->getData('isFilterable')) ? 'filters.' : 'attributes.';
//            $attribute_prefix = 'attributes.';

            $field_template = [
                "name" => $attribute_prefix.$entity->getData('code').'.value',
                "label" => $entity->getData('title'),
                "code" => $entity->getData('code'),
                "facet" => ($entity->getData('isFilterable')) ? true : false,
                "facet_size" => 100,
                "filterable" => ($entity->getData('isFilterable')) ? true : false,
                "sortable" => ($entity->getData('isSortable')) ? true : false,
                "searchable" => ($entity->getData('searchable')) ? true : false,
                "filterable_type" => "values",
                "filterable_range_step" => 0,
                "filterable_priority" => (int)$entity->getData('position'),
                "sortable_priority" => (int)$entity->getData('sort_weight'),
                "searchable_priority" => (int)$entity->getData('search_weight')
            ];

        $mapping[$entity->getData('code')] = $field_template;

        }
        return $mapping;

    }

//    private function buildFilterMapping($fields) {
//
//        $matches  = preg_grep ('/^filters.(.*).value/i', $fields);
//
//        $mapping = [];
//
//        if (count($matches)) {
//            foreach ($matches as $item) {
//
//                $attribute = explode('.',$item);
//
//                if ($attribute[1] === 'price')
//                    continue;
//
//                $field_template = [
//                    "name" => $item,
//                    "label" => ucwords($attribute[1]),
//                    "code" => $attribute[1],
//                    "facet" => true,
//                    "facet_size" => 20,
//                    "filterable" => true,
//                    "sortable" => false,
//                    "searchable" => false,
//                    "filterable_type" => "values",
//                    "filterable_range_step" => 0,
//                    "filterable_priority" => 0,
//                    "sortable_priority" => 0,
//                    "searchable_priority" => 0
////                    "sub_filters" => [
////                        [
////                            "label" => "Title",
////                            "name" => "filters." . $attribute[1] . ".title"
////                        ],
////                        [
////                            "label" => "Title",
////                            "name" => "filters." . $attribute[1] . ".image"
////                        ]
////                    ]
//                ];
//
//                $mapping[$attribute[1]] = $field_template;
//
//            }
//
//        }
//
//        return $mapping;
//
//    }

//    private function buildSearchableMapping($fields,$mapping) {
//
//        $matches  = ["attributes.name.value"];
//
//        if (count($matches)) {
//            foreach ($matches as $item) {
//
//                $attribute = explode('.',$item);
//
//                if (isset($mapping[$attribute[1]])) {
//
//                    $mapping[$attribute[1]]["searchable"] = true;
//                }
//
//                else {
//
//                    $field_template = [
//                        "name" => $item,
//                        "label" => ucwords($attribute[1]),
//                        "code"  => $attribute[1],
//                        "facet" => false,
//                        "facet_size" => 20,
//                        "filterable" => false,
//                        "sortable" => false,
//                        "searchable" => true,
//                        "filterable_type" => "values",
//                        "filterable_range_step" => 0,
//                        "filterable_priority" => 0,
//                        "sortable_priority" => 0,
//                        "searchable_priority" => 0
//                    ];
//
//                    $mapping[$attribute[1]] = $field_template;
//
//                }
//
//
//
//            }
//
//        }
//
//        return $mapping;
//
//    }

//    private function buildSortableMapping($fields,$mapping) {
//
//        $matches  = preg_grep ('/^sortables.(.*).value/i', $fields);
//
//        if (count($matches)) {
//            foreach ($matches as $item) {
//
//                $attribute = explode('.',$item);
//
//                if (isset($mapping[$attribute[1]])) {
//
//                    $mapping[$attribute[1]]["sortable"] = true;
//                }
//
//                else {
//
//                    $field_template = [
//                        "name" => $item,
//                        "label" => ucwords($attribute[1]),
//                        "code"  => $attribute[1],
//                        "facet" => false,
//                        "facet_size" => 20,
//                        "filterable" => false,
//                        "sortable" => true,
//                        "searchable" => false,
//                        "filterable_type" => "values",
//                        "filterable_range_step" => 0,
//                        "filterable_priority" => 0,
//                        "sortable_priority" => 0,
//                        "searchable_priority" => 0
//                    ];
//
//                    $mapping[$attribute[1]] = $field_template;
//
//                }
//
//
//
//            }
//
//        }
//
//        return $mapping;
//
//    }

    private function buildPriceMapping($fields,$mapping) {

        $matches  = preg_grep ('/^pricing_.(.*).final_price/i', $fields);

        if (count($matches)) {
            foreach ($matches as $item) {

                $attribute = explode('.',$item);

                $field_template = [
                    "name" => $item,
                    "label" => 'Price',
                    "code"  => $attribute[0],
                    "facet" => true,
                    "facet_size" => 20,
                    "filterable" => true,
                    "sortable" => false,
                    "searchable" => false,
                    "filterable_type" => "range",
                    "filterable_range_step" => 10,
                    "filterable_priority" => 0,
                    "sortable_priority" => 0,
                    "searchable_priority" => 0
                ];

                $mapping[$attribute[0]] = $field_template;

            }

        }

        return $mapping;

    }


    private function mapFields($index, $entities) {

        try {
            $client = $this->getHandler();
            $response = $client->request('DELETE', 'indexes/'.$index . '/mapping/field/reset', [
                'headers' => $this->getHeaders()
            ]);
            $response = $client->request('GET', 'indexes/'.$index . '/fields', [
                'headers' => $this->getHeaders()
            ]);

            $fields = $this->decodeResponse($response);

            $mapping = $this->buildMapping($index, $entities);
//            $mapping = $this->buildFilterMapping($fields);
            $mapping = $this->buildPriceMapping($fields,$mapping);
//            $mapping = $this->buildSortableMapping($fields,$mapping);
//            $mapping = $this->buildSearchableMapping($fields,$mapping);

            foreach ($mapping as $map) {

                $response = $client->request('POST', 'indexes/'.$index. '/mapping/field', [
                    'headers' => $this->getHeaders(),
                    'json' => $map
                ]);

            }

        } catch (\Exception $e) {
            echo $e->getMessage();
        }



    }

    /**
     * @param $index
     * @return $this
     */
    public function createIndex($index)
    {

        $client = $this->getHandler();
        try {
            $response = $client->request('GET', 'indexes', [
                'headers' => $this->getHeaders()
            ]);

            $indices = $this->decodeResponse($response);

            if (array_key_exists($index, $indices)) {
                return $this;
            }

            $response = $client->request('POST', 'indexes/'.$index, [
                'headers' => $this->getHeaders(),
                'json'    => [
                    "fields" => $this->base_mapping
                ]
            ]);

        } catch (\Exception $e) {
            throw new \Error($e);
        }

        return $this;
    }

    private function getHeaders() {
        return [
//            'Authorization' => 'Bearer ' . $this->token,
            'Accept'        => 'application/json',
            'X-APP-ID' => $this->username,
            'X-API-KEY' => $this->password
        ];
    }

    /**
     * @param $response
     * @return \GuzzleHttp\Psr7\Stream
     */
    private function decodeResponse($response) {
        return json_decode($response->getBody());
    }


    protected function getHandler()
    {

        $this->connect();

        return $this->handler;
    }

    /**
     * @return ConnectorInterface
     */
    public function connect(): ConnectorInterface
    {

        if (!$this->handler) {
            $hosts = $this->getHosts();

            if (!count($hosts))
                throw new \Exception('no hosts found');

            $this->handler = new \GuzzleHttp\Client(
                [
                    'base_uri' => $hosts[0]

                ]);

        }
        return $this;
    }

//    private function decodeHost($connection_string) {
//
//        $connection_parts = explode('|',$connection_string);
//
//        $auth_parts = (isset($connection_parts[0])) ? explode(":",$connection_parts[0]) : [];
//
//        $connection["host"] = (isset($connection_parts[1])) ? $connection_parts[1] : '';
//        $connection["auth"] = ['json' => [
//            "appId" => $auth_parts[0],
//            "accountId" => isset($auth_parts[1]) ? $auth_parts[1] : '',
//            "secret" => isset($auth_parts[2]) ? $auth_parts[2] : '',
//        ]];
//
//        return $connection;
//
//    }

//    /**
//     * @return array
//     */
    protected function getHosts(): array
    {
        $hosts = $this->getConfiguration()->getData(self::CONFIGURATION_HOST_POOL);

        $dsns = [];
        $dsn = '';
        foreach ($hosts as $host) {

            if ($host['username']) {
                $this->username = $host['username'];
                $this->password = $host['password'];
            }

            $dsn .= "{$host['scheme']}://";

            $dsn .= $host['host'];

            if ($host['port']) {
                $dsn .= ":{$host['port']}";
            }
            $dsns[] = $dsn;
            break;
        }

        return $dsns;
    }

    /**
     * @param array $entity
     * @param string $collection
     * @return mixed|void
     */
    public function delete(array $entity, string $collection)
    {
    }

    /**
     * @param Traversable $entities
     * @param string $collection
     * @return mixed|void
     */
    public function update(\Traversable $entities, string $collection)
    {
        $this->create($entities,$collection, 'update');
    }

    /**
     * @return array|mixed|null
     */
    protected function getPrefix()
    {
        return $this->getConfiguration()->getData('prefix');
    }
}
